@@ -14,7 +14,6 @@
  * limitations under the License.
  */
 
-/* @version $Id: java.c 1130635 2011-06-02 16:29:44Z mturk $ */
 #include "jsvc.h"
 
 #ifdef OS_CYGWIN
@@ -30,18 +29,18 @@ typedef long long __int64;
 #define jsvc_xlate_from_ascii(b) _a2e(b)
 #endif
 #else
-#define jsvc_xlate_to_ascii(b)  /* NOOP */
-#define jsvc_xlate_from_ascii(b)        /* NOOP */
+#define jsvc_xlate_to_ascii(b)         /* NOOP */
+#define jsvc_xlate_from_ascii(b)       /* NOOP */
 #endif
 
 static JavaVM *jvm = NULL;
 static JNIEnv *env = NULL;
-static jclass cls  = NULL;
+static jclass cls = NULL;
 
 #define FALSE 0
 #define TRUE !FALSE
 
-static void shutdown(JNIEnv *env, jobject source, jboolean reload)
+static void shutdown(JNIEnv * env, jobject source, jboolean reload)
 {
     log_debug("Shutdown requested (reload is %d)", reload);
     if (reload == TRUE)
@@ -50,7 +49,7 @@ static void shutdown(JNIEnv *env, jobject source, jboolean reload)
         main_shutdown();
 }
 
-static void failed(JNIEnv *env, jobject source, jstring message)
+static void failed(JNIEnv * env, jobject source, jstring message)
 {
     if (message) {
         const char *msg = (*env)->GetStringUTFChars(env, message, NULL);
@@ -103,7 +102,7 @@ char *java_library(arg_data *args, home_data *data)
     return libf;
 }
 
-typedef jint (*jvm_create_t)(JavaVM **, JNIEnv **, JavaVMInitArgs *);
+typedef jint(*jvm_create_t) (JavaVM **, JNIEnv **, JavaVMInitArgs *);
 
 bool java_signal(void)
 {
@@ -141,16 +140,16 @@ bool java_init(arg_data *args, home_data *data)
     jvm_create_t symb = NULL;
     JNINativeMethod nativemethods[2];
     JavaVMOption *opt = NULL;
-    dso_handle libh   = NULL;
+    dso_handle libh = NULL;
     JavaVMInitArgs arg;
     char *libf = NULL;
     jint ret;
     int x;
-    char loaderclass[]    = LOADER;
+    char loaderclass[] = LOADER;
     char shutdownmethod[] = "shutdown";
     char shutdownparams[] = "(Z)V";
-    char failedmethod[]   = "failed";
-    char failedparams[]   = "(Ljava/lang/String;)V";
+    char failedmethod[] = "failed";
+    char failedparams[] = "(Ljava/lang/String;)V";
     char daemonprocid[64];
     /* Decide WHAT virtual machine we need to use */
     libf = java_library(args, data);
@@ -166,8 +165,7 @@ bool java_init(arg_data *args, home_data *data)
     }
 
     /* Load the JVM library */
-#if defined(OSD_POSIX) || defined(HAVE_KAFFEVM)
-#else
+#if !defined(OSD_POSIX)
     libh = dso_link(libf);
     if (libh == NULL) {
         log_error("Cannot dynamically link to %s", libf);
@@ -181,25 +179,23 @@ bool java_init(arg_data *args, home_data *data)
     /*
        MacOS/X actually has two libraries, one with the REAL vm, and one for
        the VM startup.
-       before JVM 1.4.1 The first one (libappshell.dyld) contains CreateVM
-       JVM 1.4.1 through 1.5.* The library name is libjvm_compat.dylib
-       starting with JVM 1.6 on OS X 10.6 the library name is libverify.dylib.
+       - JVM 1.6, the library name is libverify.dylib
+       - JVM 1.7 onwards, the library name is libjli.dylib
      */
-    if (replace(appf, 1024, "$JAVA_HOME/../Libraries/libappshell.dylib",
+    if (replace(appf, 1024, "$JAVA_HOME/../Libraries/libverify.dylib",
                 "$JAVA_HOME", data->path) != 0) {
         log_error("Cannot replace values in loader library");
         return false;
     }
     if (stat(appf, &sb)) {
-        if (replace(appf, 1024, "$JAVA_HOME/../Libraries/libjvm_compat.dylib",
-                    "$JAVA_HOME", data->path) != 0) {
+        if (replace(appf, 1024, "$JAVA_HOME/../MacOS/libjli.dylib", "$JAVA_HOME", data->path) != 0) {
             log_error("Cannot replace values in loader library");
             return false;
         }
     }
+    // DAEMON-331 Alternative path for custom OpenJDK builds
     if (stat(appf, &sb)) {
-        if (replace(appf, 1024, "$JAVA_HOME/../Libraries/libverify.dylib",
-                    "$JAVA_HOME", data->path) != 0) {
+        if (replace(appf, 1024, "$JAVA_HOME/lib/jli/libjli.dylib", "$JAVA_HOME", data->path) != 0) {
             log_error("Cannot replace values in loader library");
             return false;
         }
@@ -211,13 +207,13 @@ bool java_init(arg_data *args, home_data *data)
     }
     log_debug("Shell library %s loaded", appf);
 #endif /* ifdef OS_DARWIN */
-#if defined(OSD_POSIX) || defined(HAVE_KAFFEVM)
-    /* BS2000 and kaffe does not allow to call JNI_CreateJavaVM indirectly */
+#if defined(OSD_POSIX)
+    /* BS2000 does not allow to call JNI_CreateJavaVM indirectly */
 #else
-    symb = (jvm_create_t)dso_symbol(libh, "JNI_CreateJavaVM");
+    symb = (jvm_create_t) dso_symbol(libh, "JNI_CreateJavaVM");
     if (symb == NULL) {
 #ifdef OS_DARWIN
-        symb = (jvm_create_t)dso_symbol(apph, "JNI_CreateJavaVM");
+        symb = (jvm_create_t) dso_symbol(apph, "JNI_CreateJavaVM");
         if (symb == NULL) {
 #endif /* ifdef OS_DARWIN */
             log_error("Cannot find JVM library entry point");
@@ -231,29 +227,17 @@ bool java_init(arg_data *args, home_data *data)
 
     /* Prepare the VM initialization arguments */
 
-    /*
-     * Mac OS X Java will load JVM 1.3.1 instead of 1.4.2 if JNI_VERSION_1_2
-     * is specified. So use JNI_VERSION_1_4 if we can.
-     */
-#if defined(JNI_VERSION_1_4)
-    arg.version = JNI_VERSION_1_4;
-#else
-    arg.version = JNI_VERSION_1_2;
-#endif
-#if defined(OSD_POSIX) || defined(HAVE_KAFFEVM)
-
-#if defined(HAVE_KAFFEVM)
-    memset(&arg, 0, sizeof(arg));
-    arg.version = JNI_VERSION_1_4;
-#endif
+    /* Minimum Java version is Java 6 */
+    arg.version = JNI_VERSION_1_6;
 
+#if defined(OSD_POSIX)
     if (JNI_GetDefaultJavaVMInitArgs(&arg) < 0) {
         log_error("Cannot init default JVM default args");
         return false;
     }
 #endif
     arg.ignoreUnrecognized = FALSE;
-    arg.nOptions = args->onum + 4; /* pid, ppid and abort */
+    arg.nOptions = args->onum + 5;     /* pid, ppid, version, class and abort */
     opt = (JavaVMOption *) malloc(arg.nOptions * sizeof(JavaVMOption));
     for (x = 0; x < args->onum; x++) {
         opt[x].optionString = strdup(args->opts[x]);
@@ -261,24 +245,33 @@ bool java_init(arg_data *args, home_data *data)
         opt[x].extraInfo = NULL;
     }
     /* Add our daemon process id */
-    snprintf(daemonprocid, sizeof(daemonprocid),
-             "-Dcommons.daemon.process.id=%d", (int)getpid());
+    snprintf(daemonprocid, sizeof(daemonprocid), "-Dcommons.daemon.process.id=%d", (int)getpid());
     opt[x].optionString = strdup(daemonprocid);
     jsvc_xlate_to_ascii(opt[x].optionString);
-    opt[x++].extraInfo  = NULL;
+    opt[x++].extraInfo = NULL;
+
     snprintf(daemonprocid, sizeof(daemonprocid),
              "-Dcommons.daemon.process.parent=%d", (int)getppid());
     opt[x].optionString = strdup(daemonprocid);
     jsvc_xlate_to_ascii(opt[x].optionString);
-    opt[x++].extraInfo  = NULL;
+    opt[x++].extraInfo = NULL;
+
     snprintf(daemonprocid, sizeof(daemonprocid),
              "-Dcommons.daemon.version=%s", JSVC_VERSION_STRING);
     opt[x].optionString = strdup(daemonprocid);
     jsvc_xlate_to_ascii(opt[x].optionString);
-    opt[x++].extraInfo  = NULL;
+    opt[x++].extraInfo = NULL;
+
+    /* DBCP-388. For the benefit of jconsole. */
+    snprintf(daemonprocid, sizeof(daemonprocid), "-Dsun.java.command=%s", args->clas);
+    opt[x].optionString = strdup(daemonprocid);
+    jsvc_xlate_to_ascii(opt[x].optionString);
+    opt[x++].extraInfo = NULL;
+
     opt[x].optionString = strdup("abort");
     jsvc_xlate_to_ascii(opt[x].optionString);
     opt[x].extraInfo = (void *)java_abort123;
+
     arg.options = opt;
 
     /* Do some debugging */
@@ -291,8 +284,7 @@ bool java_init(arg_data *args, home_data *data)
 
         for (x = 0; x < args->onum; x++) {
             jsvc_xlate_from_ascii(opt[x].optionString);
-            log_debug("|   \"%s\" (0x%08x)", opt[x].optionString,
-                      opt[x].extraInfo);
+            log_debug("|   \"%s\" (0x%08x)", opt[x].optionString, opt[x].extraInfo);
             jsvc_xlate_to_ascii(opt[x].optionString);
         }
         log_debug("+-------------------------------------------------------");
@@ -300,15 +292,14 @@ bool java_init(arg_data *args, home_data *data)
 
         for (; x < arg.nOptions; x++) {
             jsvc_xlate_from_ascii(opt[x].optionString);
-            log_debug("|   \"%s\" (0x%08x)", opt[x].optionString,
-                      opt[x].extraInfo);
+            log_debug("|   \"%s\" (0x%08x)", opt[x].optionString, opt[x].extraInfo);
             jsvc_xlate_to_ascii(opt[x].optionString);
         }
         log_debug("+-------------------------------------------------------");
     }
 
     /* And finally create the Java VM */
-#if defined(OSD_POSIX) || defined(HAVE_KAFFEVM)
+#if defined(OSD_POSIX)
     ret = JNI_CreateJavaVM(&jvm, &env, &arg);
 #else
     ret = (*symb) (&jvm, &env, &arg);
@@ -328,9 +319,6 @@ bool java_init(arg_data *args, home_data *data)
     }
     log_debug("Class %s found", loaderclass);
 
-#if defined(HAVE_SABLEVM)
-    log_debug("sableVM doesn't support RegisterNatives");
-#else
     jsvc_xlate_to_ascii(shutdownmethod);
     nativemethods[0].name = shutdownmethod;
     jsvc_xlate_to_ascii(shutdownparams);
@@ -347,7 +335,6 @@ bool java_init(arg_data *args, home_data *data)
         return false;
     }
     log_debug("Native methods registered");
-#endif
 
     return true;
 }
@@ -391,12 +378,12 @@ bool JVM_destroy(int exit)
 /* Call the load method in our DaemonLoader class */
 bool java_load(arg_data *args)
 {
-    jclass stringClass       = NULL;
-    jstring className        = NULL;
-    jstring currentArgument  = NULL;
+    jclass stringClass = NULL;
+    jstring className = NULL;
+    jstring currentArgument = NULL;
     jobjectArray stringArray = NULL;
-    jmethodID method         = NULL;
-    jboolean ret             = FALSE;
+    jmethodID method = NULL;
+    jboolean ret = FALSE;
     int x;
     char lang[] = "java/lang/String";
     char load[] = "load";
@@ -443,8 +430,7 @@ bool java_load(arg_data *args)
     }
 
     log_debug("Daemon loading...");
-    ret = (*env)->CallStaticBooleanMethod(env, cls, method, className,
-                                          stringArray);
+    ret = (*env)->CallStaticBooleanMethod(env, cls, method, className, stringArray);
     if (ret == FALSE) {
         log_error("Cannot load daemon");
         return false;
@@ -616,4 +602,3 @@ bool java_destroy(void)
     log_debug("Daemon destroyed successfully");
     return true;
 }
-
